//******************************************************************************
//    File: tsxAView.h
//  Module: trueSpace eXtensions API
//   Descr: Interface to the Active View
//******************************************************************************

#ifndef TSXAVIEW_H
#define TSXAVIEW_H


#include "tsxTypes.h"


//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

// This is the interface to the active viewport and its "eye" attributes.
// The Active Viewport is the currently active view into the scene.
// Eye attributes:
// - Position
// - Bank angle w.r.t. the world X axis
// - "Look at" direction
// - Zoom
// The eye's frame is similar to that of a Camera, with the Z axis pointing
// out into the scene, and the Y axis pointing up.
// See also "tsxCamra.h" and "tsxMouse.h".

//------------------------------------------------------------------------------
//	Related Types
//------------------------------------------------------------------------------

// Rendering modes
enum tsxRENDERMODE {
    e_tsxRM_UNDEF = 0,
    e_tsxRM_WIRE,   //Wire-frame
    e_tsxRM_3DR,    //3DR
    e_tsxRM_D3D     //Direct3D
};

// Viewing modes
enum tsxVIEWMODE {
    e_tsxVM_PERSPECTIVE,
    e_tsxVM_TOP,
    e_tsxVM_FRONT,
    e_tsxVM_LEFT,
    e_tsxVM_CAMERA //Look through an object or camera
};

#define tsxLIL_EOLINES		-1
	/* end of line list */
#define tsxLIL_NEWSTART		-2
	/* start of new bunch of linked lines */

enum tsxPEN {
	e_tsxPEN_BACKGROUND = 0,	/* background color */
	e_tsxPEN_GRID,				/* grid lines color */
	e_tsxPEN_ACTIVE,			/* active object color */
	e_tsxPEN_SCENE,				/* scene objects */
	e_tsxPEN_ACTIVESUB,			/* active sub object color */
	e_tsxPEN_INACTIVESUB,		/* inactive sub object color */
	e_tsxPEN_CONTROL,			/* inactive control objects */
	e_tsxPEN_ACTIVECONTROL,		/* active control objects */
	e_tsxPEN_RED,
	e_tsxPEN_GREEN,
	e_tsxPEN_BLUE
};

TSXAPIFN void tsxAViewSetPen( tsxPEN pen );

TSXAPIFN tsxBOOL tsxAViewDrawLinelist(
	CtsxTxmx3f* trans,
	CtsxVector3f* vects,
	long numVects,
	long* linelist,
	CtsxRect* rect
	);

//------------------------------------------------------------------------------
//	Refreshing the Active View
//------------------------------------------------------------------------------

// Use this function to refresh the active view of the scene after changing
// the view's attributes.
TSXAPIFN void tsxAViewRefresh();

//------------------------------------------------------------------------------
//	Refreshing All Views Simultaneously
//------------------------------------------------------------------------------

// Use this function to refresh all views of the scene simultaneously
TSXAPIFN void tsxAllViewsRefresh();

//------------------------------------------------------------------------------
//	Render and View Mode
//------------------------------------------------------------------------------

// Get the rendering mode of the active view.
TSXAPIFN tsxRENDERMODE tsxAViewGetRenderMode();

// Get the view mode for the active view.
TSXAPIFN tsxVIEWMODE tsxAViewGetViewMode();

// Change the view mode for teh active view.
// If newmode is e_tsxVM_CAMERA, it is set to look through the currently
// selected object, if any. Useful with Cameras and Lights.
// Returns: e_tsxTRUE if there is a change in mode.
TSXAPIFN tsxBOOL tsxAViewSetViewMode( tsxVIEWMODE newmode );

// Make the active view look through an object (pGNode), until further notice.
// In this mode, any changes to the eye's position, orientation and direction
// will also be transmitted to the object.
TSXAPIFN tsxERR tsxAViewSetCameraMode( tsxGNODE* pGNode );

// Turn off the camera mode, removing the constraint on the active view
// to look through an object. The current view of the scene is not changed.
TSXAPIFN void tsxAViewUnsetCameraMode();

// Get pointer to the object that the active view is currently constrained
// to look through, if any.
TSXAPIFN tsxSOBJ* tsxAViewGetCameraObj();


//------------------------------------------------------------------------------
//	Eye attributes
//------------------------------------------------------------------------------

// Get the eye position (copied into `posn') in world coordinates
TSXAPIFN void tsxAViewGetPosition( CtsxVector3f* posn );

// Get the direction in which the view is pointed, as a normalized
// vector relative to its position (copied into `dirn')
TSXAPIFN void tsxAViewGetDirection( CtsxVector3f* dirn );

// Get the Bank angle
TSXAPIFN float tsxAViewGetBankAngle();

// Get the Zoom
TSXAPIFN float tsxAViewGetZoom();

// Use this function to change the position and direction of the active
// view's "eye".  A NULL value for `pEyePosn' or `pLookAt' indicates no
// change in that parameter.
TSXAPIFN void tsxAViewSetView(
    CtsxVector3f* pEyePosn, //New position of eye, in world coordinates
    CtsxVector3f* pLookAt,  //New point at which eye should point, in world coord
    tsxBOOL bLevelIt	    //e_tsxTRUE if eye's horizontal axis should be made
			    // parallel to world X, otherwise current bank angle
			    // with world-X is maintained.
    );

// Change the bank angle between the view's horizontal axis and the world X axis.
TSXAPIFN void tsxAViewSetBank( float bankAngle );

// Change the active view's zoom (clamped to allowed min/max values).
// Returns: the new zoom value.
TSXAPIFN float tsxAViewSetZoom( float newzoom );


//------------------------------------------------------------------------------
//	Frame relationships
//------------------------------------------------------------------------------

// Matrix for transforming world coordinates to viewport (screen) coordinates.
// Returns: pTxmx.
TSXAPIFN CtsxTxmx3f* tsxAViewGetWorld2VportTxmx( CtsxTxmx3f* pTxmx );

// Matrix for transforming world coordinates to view's Eye coordinates.
// Returns: pTxmx.
TSXAPIFN CtsxTxmx3f* tsxAViewGetWorld2EyeTxmx( CtsxTxmx3f* pTxmx );

// Matrix for transforming view's Eye coordinates to world coordinates
// (inverse of the World-to-Eye matrix).
// Returns: pTxmx.
TSXAPIFN CtsxTxmx3f* tsxAViewGetEye2WorldTxmx( CtsxTxmx3f* pTxmx );

// Computes a ray pointing from the eye's viewpoint through the mouse point.
// The ray is returned in pRayOrigin and pRayDirn, the latter giving the
// direction as a normalized vector pointing from pRayOrigin.
// Any of the output arguments may be NULL, if that value is not desired.
TSXAPIFN void tsxAViewGetMouseRay(
    short mousePtX, //IN: mouse X coordinate (in screen space)
    short mousePtY, //IN: mouse Y coordinate
    CtsxVector3f* pRayOrigin, //OUT: Eye location for active view
    CtsxVector3f* pRayDirn    //OUT: Normalized direction from pRayOrigin
    );

// Computes screen cordinates of the point in the active view.
TSXAPIFN void tsxAViewGetScreenPoint(
    CtsxVector3f* pWorldPt, //IN: World coordinates
    short* pScreenPtX, //OUT: screen X coordinate (in pixels)
    short* pScreenPtY //OUT: screen Y coordinate
    );


//------------------------------------------------------------------------------
//	Picking
//------------------------------------------------------------------------------

// Pick the Root object in scene containing the object under the mouse.
// A Root object has the scene as its parent.
// Returns: e_tsxTRUE on successful pick.
TSXAPIFN tsxBOOL tsxAViewPickRoot(
    short x, //Screen coordinates of mouse
    short y,
    tsxSOBJ** ppSobj //OUT: Ptr to root object containing picked object.
    );

// Picking nearest one of MNode's vertices under the mouse, if any.
// If pMNode is 0 (null) then all objects in scene are tested.
// Returns: e_tsxTRUE on successful pick.
TSXAPIFN tsxBOOL tsxAViewPickVertex(
    tsxMNODE* pMNode, 	//Must be a Group or Polyhedron. 0 = all objs in scene.
    short x,		//Screen coordinates of mouse
    short y,
    tsxPOLYHEDRON** ppPolyh, //OUT: Polyhderon whose vertex picked
    int* pVxIndex	     //OUT: Index of picked vertex in pPolyh
    );

// Picks nearest one of MNode's faces under the mouse, if any.
// If pMNode is 0 (null) then all objects in scene are tested.
// Returns: e_tsxTRUE on successful pick.
TSXAPIFN tsxBOOL tsxAViewPickFace(
    tsxMNODE* pMNode, 	//Must be a Group or Polyhedron. 0 = all objs in scene.
    short x,		//Screen coordinates of mouse
    short y,
    tsxPOLYHEDRON** ppPolyh, //OUT: Polyhderon whose face picked
    int* pFxIndex,	     //OUT: Index of picked face in pPolyh
    float* pFxDist	     //OUT: Distance to picked face
    );


//------------------------------------------------------------------------------
//	Miscellaneous
//------------------------------------------------------------------------------

// Handle to the active view's window
TSXAPIFN HWND tsxAViewGetHwnd();


//******************************************************************************
#endif // TSXAVIEW_H
