//******************************************************************************
//    File: tsxGNode.h
//  Module: trueSpace eXtensions API
//   Descr: Definition of tsxGNODE
//******************************************************************************

#ifndef TSXGNODE_H
#define TSXGNODE_H


#include "tsxTypes.h"


enum tsxUNITS {
	e_tsxUNITS_MILLIMETERS = 0,
	e_tsxUNITS_CENTIMETERS,
	e_tsxUNITS_METERS,
	e_tsxUNITS_KILOMETERS,
	e_tsxUNITS_INCHES,
	e_tsxUNITS_FEET,
	e_tsxUNITS_YARDS,
	e_tsxUNITS_MILES,
	e_tsxUNITS_POINTS,
};

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

// A tsxGNODE is the abstract type for Nodes in a Scene Graph.
// All tsxGNODEs have position and orientation, as defined by their Axes,
// and can be visible while editing. 
// A tsxGNODE adds the following attributes to tsxSOBJ:
//      - Name
//      - parent
//      - children
//      - siblings
//      - Axes position and orientation

// NOTE on Transformations ...
//   Transformations are applied to a GNode by specifying the amount of change
//   from the current configuration.  For example, `tsxGNodeTranslate' takes a
//   translation vector as argument to specify the direction and distance of
//   translation.  This relative transformation can be given in either the World
//   frame (e.g. the translation vector is relative to the World axes) or in
//   the Model frame.  The Model frame for a GNode is specified by the position
//   and orientation of its Axes. The function `tsxGNodeGetPosition' can be used
//   to get the position of the GNode's Model-frame origin, and the function
//   `tsxGNodeGetAxes' can be used to get their orientation.  These two pieces
//   of information can be accessed together as a combined pure rotation matrix
//   through the function `tsxGNodeGetPureTransform'.
//
//   Only MNODE object types allow their Model-frame to be modified (tsxMNode.h).


//------------------------------------------------------------------------------
//	GNode type check
//------------------------------------------------------------------------------

// e_tsxTRUE if pSobj is a GNode
TSXAPIFN tsxBOOL tsxIsGNode( tsxSOBJ* pSobj );


//------------------------------------------------------------------------------
//	Management
//------------------------------------------------------------------------------

// Get a copy of the entire tree rooted at a GNode
TSXAPIFN tsxERR tsxGNodeCopy(
    tsxGNODE* pGNode,		// The GNode to copy
    tsxGNODE** ppGNodeCopy	// Ptr to Copy placed here, on success
    );


//------------------------------------------------------------------------------
//	Traverse Scene Graph
//------------------------------------------------------------------------------

// Returns: 0 	    if pGNode is the Scene object;
//          pGNode  if parent of pGNode is the Scene object,
//                  or pGNode has no parent;
//     else ptr to highest tsxGNODE ancestor of pGNode below Scene.
TSXAPIFN tsxGNODE* tsxGNodeGetRoot( tsxGNODE* pGNode );

// Get the parent node in the scene graph.
TSXAPIFN tsxGNODE* tsxGNodeGetParent( tsxGNODE* pGNode );

// Returns e_tsxTRUE if pGNode is a member of the Scene at some level.
TSXAPIFN tsxBOOL tsxIsGNodeInScene( tsxGNODE* pGNode );

// Get the first child of a Group or LOD object.
TSXAPIFN tsxGNODE* tsxGNodeGetFirstChild( tsxGNODE* pGNode );

// Get the next node of valid tsx Type
TSXAPIFN tsxGNODE* tsxGNodeGetNext( tsxGNODE* pGNode );

// Get the prev node of valid tsx Type
TSXAPIFN tsxGNODE* tsxGNodeGetPrev( tsxGNODE* pGNode );

// Returns e_tsxTRUE if pCheckGNode is a node in the tree rooted at pGNode.
TSXAPIFN tsxBOOL tsxIsGNodeSubobj( tsxGNODE* pGNode, tsxGNODE* pCheckGNode );


//------------------------------------------------------------------------------
//	Drawing/Rendering
//------------------------------------------------------------------------------

// Draws pGNode in the current view, in the current view's rendering mode.
TSXAPIFN void tsxGNodeDraw( tsxGNODE* pGNode );


//------------------------------------------------------------------------------
//	Name
//------------------------------------------------------------------------------

// Gets the Name of a GNode.
// Returns length of name. (0 if not valid request or null name)
TSXAPIFN int tsxGNodeGetName(
    tsxGNODE* pGNode,
    char*     pName,	//buffer where name is copied
    int       iNameSz	//Size of pName buffer
    );

// Sets the Name of a GNode.
// Returns 0 on success.
TSXAPIFN tsxERR tsxGNodeSetName( tsxGNODE* pGNode, char* szNewName );


//------------------------------------------------------------------------------
//	Model-frame Axes
//------------------------------------------------------------------------------

// Get the GNode's position in World coordinates.
// Returns 0 if not GNODE, else posn.
TSXAPIFN CtsxVector3f* tsxGNodeGetAxesPosition(
    tsxGNODE* pGNode,
    CtsxVector3f* posn
    );

// Get the GNode's model axes orientation, as a triple of normalized orthogonal
// vectors pointing in the directions of GNode's X, Y and Z.
// Returns 0 if not GNODE, else axes.
TSXAPIFN CtsxAxes3f* tsxGNodeGetAxesOrientation(
    tsxGNODE* pGNode,
    CtsxAxes3f* axes
    );

// Setting the axes orientation and position is only allowed for MNODEs.

// Add the bounding box of GNode to input BBox.
// Not valid for Track and Path objects.
// The resulting BBox will also enclose the input BBox bounds. To get BBox
// of only pGNode, use `tsxBBoxInit()' before calling this function.
TSXAPIFN tsxERR tsxGNodeGetBBox(
    tsxGNODE* pGNode,	//Non Track/Path GNode
    CtsxBBox3f* pBBox	//BBox updated with result.
    );

//------------------------------------------------------------------------------
//	Location, Rotation, Size
//------------------------------------------------------------------------------

// Get units of GNode. Returns e_tsxMETERS if GNode is not valid.
TSXAPIFN tsxUNITS tsxGNodeGetUnits(tsxGNODE* pGNode);

// Set units of GNode.
TSXAPIFN void tsxGNodeSetUnits(tsxGNODE* pGNode, tsxUNITS units);

// Get the location of GNode.
// Returns 0 if not GNODE, else loc.
TSXAPIFN CtsxVector3f* tsxGNodeGetLocation(
    tsxGNODE* pGNode,
    CtsxVector3f* loc
    );

// Get the euler angles of GNode's orientation.
// Returns 0 if not GNODE, else rot.
TSXAPIFN CtsxVector3f* tsxGNodeGetRotation(
    tsxGNODE* pGNode,
    CtsxVector3f* rot
    );

// Get the size of GNode.
// Returns 0 if not GNODE, else size.
TSXAPIFN CtsxVector3f* tsxGNodeGetSize(
    tsxGNODE* pGNode,
    CtsxVector3f* size
    );

// Set the location of GNode.
TSXAPIFN void tsxGNodeSetLocation(
    tsxGNODE* pGNode,
    CtsxVector3f* loc
    );

// Set the euler angles of GNode's orientation.
TSXAPIFN void tsxGNodeSetRotation(
    tsxGNODE* pGNode,
    CtsxVector3f* rot
    );

// Set the size of GNode.
TSXAPIFN void tsxGNodeSetSize(
    tsxGNODE* pGNode,
    CtsxVector3f* size
    );


//------------------------------------------------------------------------------
//	Transformation
//------------------------------------------------------------------------------

// Returns Transformation matrix representation of the GNode's axes,
//         with only Rotation and Translation elements,
//         and no Scaling/Shearing factors.
//         0 if not GNode
TSXAPIFN CtsxTxmx3f* tsxGNodeGetPureTransform(
    tsxGNODE* pGNode,
    CtsxTxmx3f* pTxmx
    );

// Returns Transformation matrix of GNode,
//         0 if not GNode
TSXAPIFN CtsxTxmx3f* tsxGNodeGetTransform(
    tsxGNODE* pGNode,
    CtsxTxmx3f* pTxmx
    );

// Translate GNode by pVec, in the specified reference frame.
TSXAPIFN void tsxGNodeTranslate(
    tsxGNODE* pGNode, 
    CtsxVector3f* pVec, // translation vector
    tsxFRAMEID frameid   // Reference frame for pVec
    );

// Rotate by fAngle radians about pAxis passing thru pCenter,
// in the specified reference frame.
TSXAPIFN void tsxGNodeRotate(
    tsxGNODE* pGNode, 
    CtsxVector3f* pCenter,	//Center of rotation.
    CtsxVector3f* pAxis,	//Rotation axis passing thru pCenter.
    float fAngle,		//Radians. Rotation angle.
    tsxFRAMEID frameid
    );

// Scale by factors along each axis, in the specified reference frame.
// Scale factors must be non-zero.
TSXAPIFN void tsxGNodeScale(
    tsxGNODE* pGNode, 
    CtsxVector3f* pScaleFacs,	//x, y, z scale factors (non-zero)
    tsxFRAMEID frameid
    );

// Scale by factor UNIFORMLY along each axis, in the specified reference frame.
// Scale factor must be non-zero.
TSXAPIFN void tsxGNodeScaleUniform(
    tsxGNODE* pGNode, 
    float fScaleFac,
    tsxFRAMEID frameid
    );


//******************************************************************************
#endif //TSXGNODE_H
