//******************************************************************************
//    File: tsxGeom.h
//  Module: trueSpace eXtensions API
//   Descr: Basic geometric manipulation in 2D and 3D.
//******************************************************************************

#ifndef TSXGEOM_H
#define TSXGEOM_H

#include <math.h>
#include "tsxTypes.h"	//Basic data types


//------------------------------------------------------------------------------
//	General comments
//------------------------------------------------------------------------------

/*
** `float' is used as the basic scalar data type.
** 
** 3D types have suffix "3f" and 2D data-types have suffix "2f".
** The functions have similar suffixes.
** 
** All normal and axis vectors used as arguments in functions are assumed to be
** unit vectors.
** 
** trueSpace uses a "Right-handed" spatial orientation.
**
** All types are defined in "tsxTypes.h".
*/

//------------------------------------------------------------------------------
//	Basics
//------------------------------------------------------------------------------

#ifndef M_PI
#define M_PI 3.14159265359
#endif

// 180/pi
#define tsxRAD2DEG_FAC 57.29577951308f
// pi/180
#define tsxDEG2RAD_FAC  0.01745329251994f

// Radians -> Degrees
inline float tsxRad2Deg( const float rad )
{
    return (rad * tsxRAD2DEG_FAC);
}

// Degrees -> Radians
inline float tsxDeg2Rad( const float deg )
{
    return (deg * tsxDEG2RAD_FAC);
}


//------------------------------------------------------------------------------
//	Vector Geometry
//------------------------------------------------------------------------------

// |vect|
TSXAPIFN float tsxMagnitude3f( const CtsxVector3f& vec );
TSXAPIFN float tsxMagnitude2f( const CtsxVector2f& vec );

//	Normalizing ... returns the length of the original vector `vec'.
TSXAPIFN float tsxNormalize3f( CtsxVector3f& vec );
TSXAPIFN float tsxNormalize2f( CtsxVector2f& vec );
// norm = vec/|vec|
TSXAPIFN float tsxGetNormalized3f( CtsxVector3f& norm, const CtsxVector3f& vec );
TSXAPIFN float tsxGetNormalized2f( CtsxVector2f& norm, const CtsxVector2f& vec );

//	Scalar-Vector arithmetic ... `vec' is modified and returned
// vec = vec + s
TSXAPIFN CtsxVector3f& tsxAddScalar3f( CtsxVector3f& vec, const float s );
// vec = vec * s
TSXAPIFN CtsxVector3f& tsxMulScalar3f( CtsxVector3f& vec, const float s );

//	Vector-Vector arithmetic ... Vector `u' is modified and returned
// u = u + v
TSXAPIFN CtsxVector3f& tsxAddVec3f( CtsxVector3f& u, const CtsxVector3f& v );
// u = u - v
TSXAPIFN CtsxVector3f& tsxSubVec3f( CtsxVector3f& u, const CtsxVector3f& v );

//	Vector Multiplication
// u . v ... Dot Product
TSXAPIFN float tsxDProdVec3f( const CtsxVector3f& u, const CtsxVector3f& v );
// uxv = u x v ... Cross Product
TSXAPIFN CtsxVector3f& tsxXProdVec3f( CtsxVector3f& uxv,
				      const CtsxVector3f& u,
				      const CtsxVector3f& v);

// Returns Signed angle from vector `u' to `v' about the axis `axis'.
// `axis' gets the (unnormalized) cross-product (axis = u x v). Angle in radians.
TSXAPIFN float tsxAngleBtwVecs3f( const CtsxVector3f& u, const CtsxVector3f& v,
				  CtsxVector3f& axis );


//------------------------------------------------------------------------------
//	Distances from a Point
//------------------------------------------------------------------------------

// Note: CtsxVector is used to represent the cartesian coordinates of a point.

//	Distances are non-negative unless mentioned otherwise.

// Distance between 2 points `p', `q'.
TSXAPIFN float tsxDistanceToPoint3f( const CtsxVector3f& p,
				     const CtsxVector3f& q );
// Distance from a point `p' to a line containing point `line_point' and
// with unit direction vector `line_unitvec'.
TSXAPIFN float tsxDistanceToLine3f( const CtsxVector3f& p,
				    const CtsxVector3f& line_point,
				    const CtsxVector3f& line_unitvec );
// Distance from a Point `p' to a Plane with normal `plane_norm' and
// containing Point `plane_point'. This distance is positive only if
// `plane_norm' points towards `p'.
TSXAPIFN float tsxDistanceToPlane3f( const CtsxVector3f& p,
				     const CtsxVector3f& plane_norm,
				     const CtsxVector3f& plane_point );


//------------------------------------------------------------------------------
//	Transformation Matrices
//------------------------------------------------------------------------------

// A = B x C ... Matrix Multiplication.
// A may be the same as B or C.
TSXAPIFN CtsxTxmx3f& tsxXProdMx3f( CtsxTxmx3f& A,
				   const CtsxTxmx3f& B, const CtsxTxmx3f& C );

// Minv = Inverse(M) ... Matrix Inverse
// Returns e_tsxFAILURE if result is not reliable.
TSXAPIFN tsxRET tsxInvertMx3f( CtsxTxmx3f& Minv, const CtsxTxmx3f& M );

// Makes Transformation Matrix from specified X and Z axes and Origin.
// pTxmx points to matrix where result Transformation Matrix is stored.
// If pInvTxmx is specified, the inverted Transform is stored there.
// Returns reference to Txmx.
TSXAPIFN CtsxTxmx3f& tsxMakeMxFromXZO3f( const CtsxVector3f& xAxis,
					 const CtsxVector3f& zAxis,
					 const CtsxVector3f& origin,
					 CtsxTxmx3f* pTxmx,
					 CtsxTxmx3f* pInvTxmx = 0 );

// Makes a transformation matrix for rotation by `angle' about a line
// defined by the center point `cent' and the normalized axis vector `axis'.
// Returns reference to Txmx.
TSXAPIFN CtsxTxmx3f& tsxMakeRotMx3f( const CtsxVector3f& cent,
				     const CtsxVector3f& axis,
				     float angle,   //radians
				     CtsxTxmx3f* pTxmx,
				     CtsxTxmx3f* pInvTxmx = 0 );
// This version builds a pure rotation matrix, leaving other cells untouched.
// The rotation is about a (unit) vector `axis'.
TSXAPIFN CtsxTxmx3f& tsxMakePureRotMx3f( const CtsxVector3f& axis,
					 float angle, //radians
					 CtsxTxmx3f* pTxmx,
					 CtsxTxmx3f* pInvTxmx = 0 );

// 	Translation in the Txmx
// Get the translation components {x, y, z} into `trvec'.
TSXAPIFN CtsxVector3f& tsxGetMxTranslation3f( CtsxVector3f& trvec,
					      const CtsxTxmx3f& Txmx );
// Set the translation components of a Txmx
TSXAPIFN CtsxTxmx3f& tsxSetMxTranslation3f( CtsxTxmx3f& Txmx,
					    const CtsxVector3f& trvec );
// Add translation vector `trvec' to Txmx
TSXAPIFN CtsxTxmx3f& tsxAddMxTranslation3f( CtsxTxmx3f& Txmx,
					    const CtsxVector3f& trvec );
// Subtract translation vector `trvec' from Txmx
TSXAPIFN CtsxTxmx3f& tsxSubMxTranslation3f( CtsxTxmx3f& Txmx,
					    const CtsxVector3f& trvec );

// Extract the pure rotation matrix (Rot) and scaling/shearing factors
// from a Transformation Matrix (Txmx).
// If specified, pScaleShear must be float[6], and is assigned as follows:
//      { Sx, Sy, Sz, Sxy, Sxz, Syz }
TSXAPIFN void tsxUnMatrix3f( const CtsxTxmx3f& Txmx,
			     CtsxTxmx3f& Rot,
			     float* pScaleShear = 0 );

//------------------------------------------------------------------------------
//	Applying the Transformation Matrix
//------------------------------------------------------------------------------

// Transform an array of vectors `pInVecs', placing result in `pOutVecs'.
TSXAPIFN void tsxTransformVecs3f( int numVecs,
				  CtsxVector3f* pOutVecs,
				  const CtsxVector3f* pInVecs,
				  const CtsxTxmx3f& Txmx );
// Apply only the rotational part of the Txmx to an array of vectors.
TSXAPIFN void tsxRotateVecs3f( int numVecs,
			       CtsxVector3f* pOutVecs,
			       const CtsxVector3f* pInVecs,
			       const CtsxTxmx3f& Txmx );

// OutAxes = Rotate InAxes using Txmx. Ignore non-rotational part of Txmx.
// IF InAxes are orthogonal and unit, then OutAxes are also valid axes.
TSXAPIFN void tsxRotateAxes3f( CtsxAxes3f* pOutAxes,
			       CtsxAxes3f* pInAxes,
			       const CtsxTxmx3f& Txmx );


//------------------------------------------------------------------------------
//	Bounding Box
//------------------------------------------------------------------------------

// Initialize BBox to negative volume.
TSXAPIFN void tsxBBoxInit( CtsxBBox3f* pBBox );

// Returns e_tsxTRUE if BBox1 and BBox2 intersect.
// If they do intersect, and pBBintx is non-zero,
// then BBintx gets their intersection.
TSXAPIFN tsxBOOL tsxBBoxIntersection(
    CtsxBBox3f* pBBintx,
    CtsxBBox3f* pBBox1,
    CtsxBBox3f* pBBox2
    );



//******************************************************************************
#endif // TSXGEOM_H
