//******************************************************************************
//    File: tsxLight.h
//  Module: trueSpace eXtensions API
//   Descr: Interface to tsxLIGHT
//******************************************************************************

#ifndef TSXLIGHT_H
#define TSXLIGHT_H


#include "tsxTypes.h"


//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

// tsxLIGHT contains the following information in addition to tsxGNODE:
//	- type
//      - color
//      - intensity
//      - falloff
//      - shadow properties
//      - (Spot) cone angle, hot-spot ratio
// Please consult the trueSpace Reference Manual for detailed description
// of the shadow and other attributes.
// The Z axis of a Spot or Infinite Light points in the direction of the
// light rays.


//------------------------------------------------------------------------------
//	Related types and definitions
//------------------------------------------------------------------------------

// Light types
enum tsxLIGHT_TYPE {
    e_tsxLT_UNDEF = 0,
    e_tsxLT_INFINITE, //Light from an very distant source
    e_tsxLT_LOCAL,    //Light from a local point source
    e_tsxLT_SPOT	    //A spot-light
};

// Light intensity falloff
enum tsxLIGHT_FALLOFF {
    e_tsxLF_UNDEF = 0,
    e_tsxLF_ZERO,    //No falloff
    e_tsxLF_LINEAR,  //Linear with distance
    e_tsxLF_SQUARED  //square of distance
};

// Shadow type
enum tsxLIGHT_SHADOWING {
    e_tsxLS_NOSHADOWS = 0,
    e_tsxLS_RAYTRACE,
    e_tsxLS_SHADOWMAP
};

// Shadow-map Size
enum tsxLIGHT_SMSIZE {
    e_tsxSMZ_UNDEF = 0,
    e_tsxSMZ_LOW,
    e_tsxSMZ_MEDIUM,
    e_tsxSMZ_HIGH
};

// Shadow-map sharpness
enum tsxLIGHT_SMSHARPNESS {
    e_tsxSMS_UNDEF = 0,
    e_tsxSMS_LOW,
    e_tsxSMS_MEDIUM,
    e_tsxSMS_HIGH
};


//------------------------------------------------------------------------------
//	Managers
//------------------------------------------------------------------------------

// Creates a Light of specified LtType at world origin,
// with default position, orientation and attributes.
// On success, the new Light is the currently selected object.
// Default location: Spot, Local -- (0,0,1), Infinite -- (0,0,0).
// Default orientation: 180 deg rotation about world X, making it
//  point towards world origin (spot, infinite) with its Z axis.
//  The axes are aligned as follows:
//    X aligned with World X,
//    Y and Z aligned with world negative Y and negative Z.
TSXAPIFN tsxERR tsxLightCreateInScene( tsxLIGHT_TYPE LtType );

// Use `tsxSobjDelete' to remove and delete from scene, and GNODE functions
// to manage in the scene graph and access/modify position.


//------------------------------------------------------------------------------
//	Common Attributes
//------------------------------------------------------------------------------

// Note: None of the following routines for modifying a light's attributes
// update the display.

// Light type
TSXAPIFN tsxLIGHT_TYPE tsxLightGetType( tsxLIGHT* pLight );

// Get Light color (copied into pColor).
TSXAPIFN void tsxLightGetColor( tsxLIGHT* pLight, CtsxColor* pColor );

// Set Light color (copied from pColor).
TSXAPIFN void tsxLightSetColor( tsxLIGHT* pLight, CtsxColor* pColor );

// Get the light intensity. Returns -1 if not a light.
TSXAPIFN float tsxLightGetIntensity( tsxLIGHT* pLight );

// Set the light intensity. Returns new intensity, -1.0 if not a light.
TSXAPIFN float tsxLightSetIntensity( tsxLIGHT* pLight, float fNewIntens );

// Get the light intensity falloff type with distance.
TSXAPIFN tsxLIGHT_FALLOFF tsxLightGetFalloff( tsxLIGHT* pLight );

// Set the light intensity falloff type with distance.
// Returns new value, or tsxLF_UNDEF for invalid request.
TSXAPIFN tsxLIGHT_FALLOFF tsxLightSetFalloff( tsxLIGHT* pLight,
					      tsxLIGHT_FALLOFF NewFo );

// Shadowing.
TSXAPIFN tsxLIGHT_SHADOWING tsxLightGetShadType( tsxLIGHT* pLight );

// Set shadowing type.
// Returns new value, e_tsxLS_NOSHADOWS if not a light.
TSXAPIFN tsxLIGHT_SHADOWING tsxLightSetShadType(
    tsxLIGHT* pLight,
    tsxLIGHT_SHADOWING newShType
    );

// True if shadow-map is image size dependent.
TSXAPIFN tsxBOOL tsxLightIsShadImgDep( tsxLIGHT* pLight );

// (Un)Set shadow-map's image size dependence.
// Returns new value, False if not a light.
TSXAPIFN tsxBOOL tsxLightSetShadImgDep(
    tsxLIGHT* pLight,
    tsxBOOL bDependent //True to make it dependent
    );

// Get shadow-map size
TSXAPIFN tsxLIGHT_SMSIZE tsxLightGetShmapSize( tsxLIGHT* pLight );

// Set shadow-map size. Returns new value, or e_tsxSMZ_UNDEF if invalid request.
TSXAPIFN tsxLIGHT_SMSIZE tsxLightSetShmapSize(
    tsxLIGHT* pLight,
    tsxLIGHT_SMSIZE NewSz
    );

// Get shadow-map sharpness (_UNDEF if not light).
TSXAPIFN tsxLIGHT_SMSHARPNESS tsxLightGetShmapSharpness( tsxLIGHT* pLight );

// Set shadow-map sharpness.
// Returns new value, or tsxSMS_UNDEF for invalid request.
TSXAPIFN tsxLIGHT_SMSHARPNESS tsxLightSetShmapSharpness(
    tsxLIGHT* pLight,
    tsxLIGHT_SMSHARPNESS NewSharpness
    );


//------------------------------------------------------------------------------
//	Spotlight Attributes
//------------------------------------------------------------------------------

// Get cone angle (in radians).
// This is the solid angle of the lit area subtended at the light source.
// Returns -1.0 if not a spotlight.
TSXAPIFN float tsxLightGetSpotAngle( tsxLIGHT* pLight );

// Set the spotlight cone angle (in radians).
// Returns the new angle, -1 if not a spotlight.
TSXAPIFN float tsxLightSetSpotAngle( tsxLIGHT* pLight, float newAngle );

// Get the hot-spot ratio.
// The hot-spot in a spotlight is the inner circle of even bright light.
// In the lit ring outside the hot-spot, the light intensity decreases
// towards the outer edge. The hot-spot ratio is the ratio of the hot-spot
// cone solid angle to the entire spotlight cone solid angle.
// Returns 0.0 <= ratio <= 1.0, -1.0 if not a light.
TSXAPIFN float tsxLightGetSpotRatio( tsxLIGHT* pLight );

// Set the hot-spot ratio (see `tsxLightGetSpotRatio').
// Returns new value, -1.0 if not a light, or invalid newRatio.
TSXAPIFN float tsxLightSetSpotRatio( tsxLIGHT* pLight, float newRatio );


//******************************************************************************
#endif // TSXLIGHT_H
