//******************************************************************************
//    File: tsxMatrl.h
//  Module: trueSpace eXtensions API
//   Descr: Definition of tsxMATERIAL
//******************************************************************************

#ifndef TSXMATRL_H
#define TSXMATRL_H


#include "tsxTypes.h"


//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

// tsxMATERIAL is used to set the material for obects and faces.
// Use the following structures and functions to access, update and create
// materials.  Functions for painting with materials are in "tsxPolyh.h".

//------------------------------------------------------------------------------
//	Related Types and Values
//------------------------------------------------------------------------------

// Solid texture direction values (used in the grain fields).
#define tsxSOLID_DIR_X  0
#define tsxSOLID_DIR_Y  1
#define tsxSOLID_DIR_Z  2

// Simulates a granite like texture with upto 4 colors.
struct CtsxGraniteProps
{
    CtsxColor color1; 	//The different stone colors.
    CtsxColor color2;
    CtsxColor color3;
    CtsxColor color4;
    float amount1;	//Relative amount of color1 [0.0 - 1.0] (0=none)
    float amount2;	//Relative amount of color2 [0.0 - 1.0]
    float amount3;	//Relative amount of color3 [0.0 - 1.0]
    float amount4;	//Relative amount of color4 [0.0 - 1.0]
    float sharpness;	//[0.0 (max blurring to one color) - 1.0 (no blur)]
    CtsxVector3f scale;	//Scaling along solid texture directions. Non-negative.
			// Larger values result in finer texture. 
    unsigned short seed; //Random seed for generating pattern [0 - 64k]
};

// Simulates texture of swirling veins produced by turbulent forces
// applied before marble solidifies.
struct CtsxMarbleProps
{
    CtsxColor stonecol;	//Base stone color
    CtsxColor veincol;	//Color of veins
    short turbulence;	//Higher numbers result in larger, bumpier veins.
			// 0 produces straight veins. Typically [0 - 20].
    float sharpness;	//Blending between vein and stone colors at vein edges.
			// Higher values increase sharpness. [0.0 - 1.0]
    short grain;	//Vein direction (tsxSOLID_DIR_*).
    CtsxVector3f scale; //Scaling along solid texture directions. Non-negative.
			// Larger values result in more veins.
    unsigned short seed; //Random seed for generating pattern [0 - 64k]
};

// This produces concentric light (Spring) and dark (Summer) colored rings,
// around an axis.
struct CtsxWoodProps
{
    CtsxColor lightcol; //The light or spring color
    CtsxColor darkcol;  //The dark or summer color
    float ldratio;	//Light/Dark ratio.
			// [0.0 (almost all dark) - 1.0 (almost all light)]
    float rdensity;	//Number of rings per unit distance.
    float rwidthvar;	//Variation in ring width [0.0 (uniform) - 1.0]
    float rshapevar;	//Variation in ring shape [0.0 (perfect circle) - 1.0]
    CtsxVector3f center; //Location of rings center.
    short grain;	//Ring axis direction (tsxSOLID_DIR_*).
    CtsxVector3f scale;	//Scaling along solid texture directions. Non-negative.
    unsigned short seed; //Random seed for generating pattern [0 - 64k]
};

// Values for CtsxProcTexture::txtrtype
#define tsxTEXTURE_NONE	   0
#define tsxTEXTURE_GRANITE 1
#define tsxTEXTURE_MARBLE  2
#define tsxTEXTURE_WOOD    3
// returned by `tsxMaterialGetTextureType' if texture is not procedural.
// Not a valid value for field txtrtype.
#define tsxTEXTURE_IMAGE   4

// Structure for specifying one of the procedural textures in tsxMATERIAL
struct CtsxProcTexture
{
    short txtrtype; //one of tsxTEXTURE_*
    union
    {
	CtsxGraniteProps gprops;
	CtsxMarbleProps  mprops;
	CtsxWoodProps    wprops;
    };
};

// Reflection shading types (mutually exclusive)
#define tsxSHADER_FLAT	0x01
#define tsxSHADER_PHONG	0x02
#define tsxSHADER_METAL	0x04
// Surface smoothness (mutually exclusive)
#define tsxFACET_FACETED 0x01
#define tsxFACET_AUTO	 0x02
#define tsxFACET_SMOOTH	 0x04

// Surface properties ......................................
struct CtsxSurfaceProps
{
    float ka;	//ambient coefficent [0.0 -(0.01)- 1.0]
    float ks;	//specular coefficent [0.0 -(0.01)- 1.0]
    float exp;	//exponent of specularity (roughness) [0.0 -(0.01)- 1.0]
};

// Texture Map properties ..................................

#define tsxTXTR_OVERLAY	0x0001
#define tsxTXTR_ON	0x0002  //otherwise OFF
#define tsxTXTR_MOVIE	0x0004
#define tsxTXTR_MARBLE  0x0400
#define tsxTXTR_WOOD    0x0800
#define tsxTXTR_GRANITE 0x1000
// Only one of MARBLE, WOOD or GRANITE may be set.

struct CtsxTextureProps
{
    float offsetu;	  //texture U offset [-1.0 -(.001)- 1.0]
    float offsetv;	  //texture V offset [-1.0 -(.001)- 1.0]
    float repeatsu;  	  //texture repeats in U [0.01 -(.1)- 100.0]
    float repeatsv;  	  //texture repeats in V [0.01 -(.1)- 100.0]
    unsigned short flags; //see above
};

// Bump Map properties .....................................

#define tsxBUMP_ON	0x0002  //otherwise OFF
#define tsxBUMP_MOVIE	0x0004

struct CtsxBumpProps
{
    float offsetu;	//bump U offset [-1.0 -(.001)- 1.0]
    float offsetv;	//bump V offset [-1.0 -(.001)- 1.0]
    float repeatsu;	//bump repeats in U [0.01 -(.1)- 100.0]
    float repeatsv;	//bump repeats in V [0.01 -(.1)- 100.0]
    float amplitude;	//bump amplitude [-10.0 - 10.0]
    unsigned short flags;	//see above
};

// Environment Map properties ..............................

#define tsxENVR_CUBIC	0x0001  //cubic environment, otherwise its spherical
#define tsxENVR_ON	0x0002  //otherwise OFF
#define tsxENVR_MOVIE	0x0004


#define tsxUVPROJ_PLANE		0x01
#define tsxUVPROJ_PLANE_SQUARE	0x02
#define tsxUVPROJ_CYLINDER	0x04
#define tsxUVPROJ_SPHERE	0x08


//------------------------------------------------------------------------------
//	Managers 
//------------------------------------------------------------------------------

// Create a new material object with default properties:
//   - reflection shading = tsxSHADER_FLAT
//   - Surface smoothness = tsxFACET_FACETED
//   - alpha-chanel = 1
//   - index of refraction = 1
//   - Texture U repeats = 1
//   - Texture V repeats = 1
//   - Bump U repeats = 1
//   - Bump V repeats = 1
//   - Bump amplitude = 1
//   - Zero (0) in the rest of the fields
// Ptr to new object assigned to ppMatrl on success.
TSXAPIFN tsxERR tsxMaterialCreate( tsxMATERIAL** ppMatrl );

// Create a new material object, with same properties as pOldMatrl.
// Ptr to new object assigned to ppNewMatrl on success.
TSXAPIFN tsxERR tsxMaterialCreateCopy(
    tsxMATERIAL** ppNewMatrl,
    tsxMATERIAL* pOldMatrl
    );

// Create a new material object, with same properties as the Active Material.
// Ptr to new object assigned to ppMatrl on success.
TSXAPIFN tsxERR tsxMaterialCreateCopyActive( tsxMATERIAL** ppMatrl );

// Destroy a material object.  If assigned to faces, corresponding material
// entries in matlist set to null.
TSXAPIFN void tsxMaterialDestroy( tsxMATERIAL* pMatrl );


//------------------------------------------------------------------------------
//	Active Material
//------------------------------------------------------------------------------

// There is an active material which in trueSpace is the material last painted,
// and the default material for the next painting action.

// Set the active material to pNewActive (may be NULL).
TSXAPIFN void tsxMaterialSetActive( tsxMATERIAL* pNewActive );

// Returns ptr to the Active Material (may be NULL)
TSXAPIFN tsxMATERIAL* tsxMaterialGetActive();

// Copy Active Material's (if any) attributes into pMatrl.
TSXAPIFN void tsxMaterialCopyActive( tsxMATERIAL* pMatrl );


//------------------------------------------------------------------------------
//	Attribute Access
//------------------------------------------------------------------------------

// Testing two materials for equivalence.
TSXAPIFN tsxBOOL tsxMaterialsAreEqual( tsxMATERIAL* pMatrl1,
				       tsxMATERIAL* pMatrl2 );

// Get its ID (0-64K, -1 if not material)
TSXAPIFN int tsxMaterialGetId( tsxMATERIAL* pMatrl );


// Surface attributes ..........................................................

// Reflection shading (tsxSHADER_*, 0 on error)
TSXAPIFN int tsxMaterialGetShadingType( tsxMATERIAL* pMatrl );

// Surface faceting (tsxFACET_*, 0 on error)
TSXAPIFN int tsxMaterialGetFacetingType( tsxMATERIAL* pMatrl );

// The angle in degrees (0 - 120)
TSXAPIFN int tsxMaterialGetAutofacetAngle( tsxMATERIAL* pMatrl );


// Color
TSXAPIFN void tsxMaterialGetColor( tsxMATERIAL* pMatrl, CtsxColor* pColor );

// Surface properties
TSXAPIFN void tsxMaterialGetSurfaceProps( tsxMATERIAL* pMatrl,
					  CtsxSurfaceProps* pSprops );

// index of refraction (1.0 - 2.0, 0 on error)
TSXAPIFN float tsxMaterialGetIor( tsxMATERIAL* pMatrl );


// Texture Map .................................................................

// Returns tsxTEXTURE_*
TSXAPIFN int tsxMaterialGetTextureType( tsxMATERIAL* pMatrl );

// If not procedural texture, pPrTxt->txtrtype = tsxTEXTURE_NONE.
TSXAPIFN void tsxMaterialGetProcTexture( tsxMATERIAL* pMatrl,
					 CtsxProcTexture* pPrTxt );

// File used as texture map image.
TSXAPIFN void tsxMaterialGetTextureFilename(
    tsxMATERIAL* pMatrl,
    char* szFilename,	// File name copied here, if any
    int iFilenameSize	// size of szFilename buffer
    );

// Texture mapping parameters
TSXAPIFN void tsxMaterialGetTextureProps(
    tsxMATERIAL* pMatrl,
    CtsxTextureProps* pTxtrProps
    );


// Bump Map ....................................................................

// File used as bump map image.
TSXAPIFN void tsxMaterialGetBumpFilename(
    tsxMATERIAL* pMatrl,
    char* szFilename,	// File name copied here, if any
    int iFilenameSize	// size of szFilename buffer
    );

// Bump mapping parameters
TSXAPIFN void tsxMaterialGetBumpProps(
    tsxMATERIAL* pMatrl,
    CtsxBumpProps* pBumpProps
    );


// Environment Map .............................................................

// File used as environment map image.
TSXAPIFN void tsxMaterialGetEnvrFilename(
    tsxMATERIAL* pMatrl,
    char* szFilename,	// File name copied here, if any
    int iFilenameSize	// size of szFilename buffer
    );


// Returns tsxENVR_* (0 on error)
TSXAPIFN int tsxMaterialGetEnvrFlags( tsxMATERIAL* pMatrl );


//------------------------------------------------------------------------------
//	Attribute Update
//------------------------------------------------------------------------------

// Surface attributes ..........................................................

// Reflection shading (tsxSHADER_*)
TSXAPIFN void tsxMaterialSetShadingType( tsxMATERIAL* pMatrl, int shading );

// Surface faceting (tsxFACET_*)
TSXAPIFN void tsxMaterialSetFacetingType( tsxMATERIAL* pMatrl, int faceting );

// The angle in degrees (0 - 120)
TSXAPIFN void tsxMaterialSetAutofacetAngle( tsxMATERIAL* pMatrl,
					    int angleDegrees );


// Color
TSXAPIFN void tsxMaterialSetColor( tsxMATERIAL* pMatrl, CtsxColor* pColor );

// Surface properties
TSXAPIFN void tsxMaterialSetSurfaceProps( tsxMATERIAL* pMatrl,
					  CtsxSurfaceProps* pSprops );

// index of refraction (1.0 - 2.0)
TSXAPIFN void tsxMaterialSetIor( tsxMATERIAL* pMatrl, float ior );


// Texture Map .................................................................

// Returns tsxTEXTURE_*
TSXAPIFN void tsxMaterialSetTextureType( tsxMATERIAL* pMatrl, int txtype );

// If not procedural texture, pPrTxt->txtrtype = tsxTEXTURE_NONE.
TSXAPIFN void tsxMaterialSetProcTexture( tsxMATERIAL* pMatrl,
					 CtsxProcTexture* pPrTxt );

// File used as texture map image.
TSXAPIFN void tsxMaterialSetTextureFilename(
    tsxMATERIAL* pMatrl,
    char* szFilename	// File name ptr (may be NULL)
    );

// Texture mapping parameters
TSXAPIFN void tsxMaterialSetTextureProps(
    tsxMATERIAL* pMatrl,
    CtsxTextureProps* pTxtrProps
    );


// Bump Map ....................................................................

// File used as bump map image.
TSXAPIFN void tsxMaterialSetBumpFilename(
    tsxMATERIAL* pMatrl,
    char* szFilename	// File name ptr (may be 0)
    );

// Bump mapping parameters
TSXAPIFN void tsxMaterialSetBumpProps(
    tsxMATERIAL* pMatrl,
    CtsxBumpProps* pBumpProps
    );


// Environment Map .............................................................

// File used as environment map image.
TSXAPIFN void tsxMaterialSetEnvrFilename(
    tsxMATERIAL* pMatrl,
    char* szFilename	// File name ptr (may be 0)
    );

// Environment flags tsxENVR_*
TSXAPIFN void tsxMaterialSetEnvrFlags( tsxMATERIAL* pMatrl, int flags );


//******************************************************************************
#endif //TSXMATRL_H
