//******************************************************************************
//    File: tsxMouse.h
//  Module: trueSpace eXtensions API
//   Descr: Interface to the Mouse
//******************************************************************************

#ifndef TSXMOUSE_H
#define TSXMOUSE_H


#include "tsxTypes.h"


//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

// This is the interface to the Mouse-Tool in trueSpace.
// For an extension to receive mouse action messages, it must install
// the mouse-tool (using tsxMtoolInstall).
// See also picking functions in "tsxAView.h".

//------------------------------------------------------------------------------
//	Related Types and Definitions
//------------------------------------------------------------------------------

// Internal data structure for managing the mouse tool.
struct tsxMousetool {};

// Callback type for mouse-tool message
typedef BOOL (*tsxMtoolMsg)(
    tsxMousetool *mt,
    short msg, 	//see mouse-tool messages, below.
    short x,   	//mouse position on screen, viewport X coordinate.
    short y,   	//mouse position on screen, viewport Y coordinate.
    short data 	//additional data depending upon msg.
    );
// Callback type for mouse events
typedef BOOL (*tsxMtoolClick)(
    tsxMousetool *mt,
    short x,   	//mouse position on screen, viewport X coordinate.
    short y,   	//mouse position on screen, viewport Y coordinate.
    short flags //see mouse event flags, below.
    );

// Mouse-tool messages
#define tsxMTM_ACTIVATE	  1 //Mousetool is being activated.
#define tsxMTM_DEACTIVATE 2 //Mousetool is being deactivated.
#define tsxMTM_DESELECT	  3 //Current object is being deselected.
#define tsxMTM_SELECT	  4 //New current object has been selected.
#define tsxMTM_KEY	  5 //Key has been pressed: char is in `data'.
                            // Return FALSE if key is used.
#define tsxMTM_CREATE	  6 //Mousetool has been created.
#define tsxMTM_DESTROY	  7 //Mousetool is being destroyed.
#define tsxMTM_ERASE	  8 //Erase requested on currently selected object.
			    // If TRUE is returned the current object is deleted.
#define tsxMTM_CHECKCURSOR 9 //Called on WM_MOUSEMOVE, WM_KEYUP, WM_KEYDOWN.
			     // `data' contains mouse event flags.
			     // For Mouse-move, mouse coordinates are passed.

// Mouse event flags
#define tsxMTF_RIGHT     0x01 //Right mouse button clicked
#define tsxMTF_LEFT      0x02 //Left mouse button clicked
#define tsxMTF_SHIFT     0x04 //Shift key is pressed
#define tsxMTF_CTRL	 0x08 //Control key is pressed
#define tsxMTF_OUTSIDE 0x4000 //Mouse is outside window (for drag operations)
#define tsxMTF_DBLCLK  0x8000 //Mouse button was double-clicked


//------------------------------------------------------------------------------
//	Management
//------------------------------------------------------------------------------

// When a new mouse-tool is installed, `pMtoolMsgFunc' is called with the
// message tsxMTM_CREATE, and then with tsxMTM_ACTIVATE. The eXtension
// MUST be active at this time.
// A mouse-tool installed by an extension is removed before the extension is
// deactivated. It is also removed when trueSpace receives a new install request.
// Before a mouse-tool is removed, its message function `pMtoolMsgFunc' is
// called first with the message tsxMTM_DEACTIVATE and then with tsxMTM_DESTROY.
// 
// Typical sequence of events:
//    - User selects eXtn
//    - eXtn is activated
//    - eXtn installs mouse-tool
//    - pMtoolMsgFunc called with message tsxMTM_CREATE
//    - pMtoolMsgFunc called with message tsxMTM_ACTIVATE
//    - ... handle mouse events ...
//    - user selects another tS tool
//    - pMtoolMsgFunc called with message tsxMTM_DEACTIVATE
//    - pMtoolMsgFunc called with message tsxMTM_DESTROY
//    - eXtn is deactivated (tsxDeactivate)
//    - control switches to the new tool selected by user

// Installs the mouse-tool if the eXtension is active.
// Function pointers cannot be NULL.
TSXAPIFN tsxERR tsxMtoolInstall(
    int tsxid,			   // extension-id (see tsxGetData)
    tsxMtoolMsg pMtoolMsgFunc,	   // mouse-tool-message callback
    tsxMtoolClick pMtoolEventFunc  // mouse-event callback
    );

// Removes the mouse-tool installed by this extension, if any.
// Extension must be active.
TSXAPIFN void tsxMtoolRemove( int tsxid );


//******************************************************************************
#endif // TSXMOUSE_H
