//******************************************************************************
//    File: tsxSelect.h
//  Module: trueSpace eXtensions API
//   Descr: The Selection Mechanism
//******************************************************************************

#ifndef TSXSELECT_H
#define TSXSELECT_H

#include "tsxTypes.h"


//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

// This describes the API to trueSpace's selection mechanism. Several trueSpace
// functions require their target object to be currently selected.
// Selectable objects are:
//    - GNodes
//    - Axes
// trueSpace may allow selection of some unsupported object types (which
// show up as e_tsxUNDEFINED). Call the appropriate Selection function
// (e.g. tsxSelectNext) until a TSX-supported Node type is selected.


//------------------------------------------------------------------------------
//	Select Modes
//------------------------------------------------------------------------------

enum tsxSELECTMODE {
    e_tsxSELECT,	// Ordinary Selection of a Node.
    e_tsxERASE,		// For selecting an object after erasing curr obj.
    e_tsxGLUE		// For selecting an object to attach to curr obj.
};


//------------------------------------------------------------------------------
//	Tests
//------------------------------------------------------------------------------

// e_tsxTRUE if pSobj is of supported selectable type (see doc above).
TSXAPIFN tsxBOOL tsxIsSelectable( tsxSOBJ* pSobj );

// e_tsxTRUE if pSobj is part of a selected Object/Group
TSXAPIFN tsxBOOL tsxIsSelected( tsxSOBJ* pSobj );


//------------------------------------------------------------------------------
//	Accessing/Making/Changing a Selection
//------------------------------------------------------------------------------

// Get pointer to the currently selected object.
// Note: trueSpace may select objects that are not currently supported in TSX.
TSXAPIFN tsxSOBJ* tsxGetCurrentSelection();

// Essentially same (with correct args) as picking an object with the mouse.
// Make specified pSobj the current selection.
// Set `bNoDraw' to e_tsxTRUE if you do not wish the object to be redrawn in
// "selected" mode.  Use this feature carefully, as it might result in all
// or portions of the object being erased when the next object is selected.
// Primarily meant as a mechanism for temporarily selecting an object for
// operations, and to be followed by a scene refresh.
TSXAPIFN void tsxSelectSobj(
    tsxSOBJ* pSobj,		//Must be `tsxIsSelectable'
    tsxSELECTMODE selmode,
    tsxBOOL bNoDraw		//TRUE if Views should not be updated.
    );

// Same function as called on the [downArrow] button.
// Selects a child of the Currobj.
// Views updated.
TSXAPIFN void tsxSelectDown();

// Same function as called on the [upArrow] button.
// Selects parent of Currobj.
// Views updated.
TSXAPIFN void tsxSelectUp();

// Same function as called on the <rightArrow> key.
// Selects a sibling of the Currobj, following Currobj in the list.
// Views updated.
TSXAPIFN void tsxSelectNext();

// Same function as called on the <leftArrow> key.
// Selects a sibling of the Currobj, preceeding Currobj in the list.
// Views updated.
TSXAPIFN void tsxSelectPrev();


//------------------------------------------------------------------------------
//	Callback
//------------------------------------------------------------------------------

// The function `func' will get called each time there is a change in the
// current selection, made by the normal trueSpace mouse tool or through a
// call to one of the above functions for making/changing a selection, while
// an eXtension is active. Use 0 (NULL) to remove an installed callback.
// Installation and callback works only while an eXtension is active.
// Returns the previous callback function pointer (or 0 if none).
TSXAPIFN tsxBasicCallbackFP tsxSelectionChangedCB(
    int tsxid,			// Id of this eXtn (see tsxGetData).
    tsxBasicCallbackFP func	// This function gets installed as the callback.
    );


//------------------------------------------------------------------------------
//	Misc fns dealing with Currobj
//------------------------------------------------------------------------------

// Removes the current object and substitutes it with pNewGNode.
// If the selected part of the current object does not include it's children,
// the children are moved to the new object and removed from the current
// object, leaving the children non-selected. pNewGNode is installed in
// Currobj's location, and becomes the new Currobj. The old Currobj is deleted.
TSXAPIFN void tsxCurrobjReplace(
    tsxGNODE* pNewGNode,
    tsxBOOL bNoDraw	   //TRUE if View displays should not be refreshed.
    );

// Copy curr obj and add to scene at top level, making it new currobj.
TSXAPIFN tsxERR tsxCurrobjCopy();

// Draw the curr selection
TSXAPIFN void tsxCurrobjDraw();

// Photo-render the current object in the active view.
TSXAPIFN void tsxCurrobjPhrender();


//******************************************************************************
#endif //TSXSELECT_H
