//******************************************************************************
//    File: tsxTypes.h
//  Module: trueSpace eXtensions API
//   Descr: Basic data types used in most places
//******************************************************************************

#ifndef TSXTYPES_H
#define TSXTYPES_H

//------------------------------------------------------------------------------
//	Misc
//------------------------------------------------------------------------------

// Return type for functions that test a condition.
enum tsxBOOL { e_tsxFALSE = 0, e_tsxTRUE };

// Return type for functions that simply report success or failure.
enum tsxRET { e_tsxFAILURE = 0, e_tsxSUCCESS };

// Return type for functions that return an error code when they fail.
// These are documented in "tsxErr.h".
typedef int tsxERR;


//------------------------------------------------------------------------------
//	Geometry
//------------------------------------------------------------------------------

// Reference Frames: World or Model
enum tsxFRAMEID { e_tsxWorldFrame, e_tsxModelFrame };


// Note:
// trueSpace uses `float' to represent all coordinates and related values.
// 3D types have suffix "3f" and 2D data-types have suffix "2f".


// Specifying Vectors and Points
struct CtsxVector2f
{
    float x, y;
};
struct CtsxVector3f
{
    float x, y, z;
};

// Specifying axes.
struct CtsxAxes3f
{
    CtsxVector3f xaxis;  //Unit vector specifying orientation of Model's X axis.
    CtsxVector3f yaxis;  // ... Y ...
    CtsxVector3f zaxis;  // ... Z ...
};

// Transformation Matrix.
//  This is the traditional homogeneous matrix, with the columns and rows
//  reversed from what is customary in algebra notation for 3D operations.
//  Most 3D algebra is based on a 4 x 4 matrix, with the terms in the last
//  column always {0,0,0,1}. In our representation this would be the last
//  row, which is kept implicit and not stored.
struct CtsxTxmx3f
{
    float matrix[3][4];
};

// Bounding Box.
//  This is a (smallest) world-axes aligned paralellopiped enclosing an object.
//  Since it is aligned with the world axes, the box is definable through its
//  minimum and maximum coordinates.
struct CtsxBBox3f
{
    CtsxVector3f minbounds;	// Minimum X,Y,Z coordinates.
    CtsxVector3f maxbounds;	// Maximum X,Y,Z coodinates.
};


//------------------------------------------------------------------------------
//	Geometry Inlined Functions
//------------------------------------------------------------------------------

// Equality test on CtsxVector3f
inline BOOL operator==( const CtsxVector3f& vec1, const CtsxVector3f& vec2 )
{
    return ((vec1.x == vec2.x) &&
	    (vec1.y == vec2.y) &&
	    (vec1.z == vec2.z));
}

// Equality test on CtsxVector2f
inline BOOL operator==( const CtsxVector2f& vec1, const CtsxVector2f& vec2 )
{
    return ((vec1.x == vec2.x) &&
	    (vec1.y == vec2.y));
}


//------------------------------------------------------------------------------
//	Color, UV-Space
//------------------------------------------------------------------------------

// For specifying color, e.g. in a material or for a vertex.
struct CtsxColor
{
    unsigned char red;	 // 0 - 255
    unsigned char green;
    unsigned char blue;
    unsigned char alpha; // 0 - 100
};

// For specifying UV-space coordinates of a vertex.
struct CtsxUV
{
    float u;
    float v;
};


//------------------------------------------------------------------------------
//	Picking rectangle
//------------------------------------------------------------------------------

struct CtsxRect {
	short x, y;
	short MaxX, MaxY;
};

//------------------------------------------------------------------------------
//	Scene Graph
//------------------------------------------------------------------------------

// The Scene graph is arranged as follows:
//    There is a single tsxSCENE object at the top.
//       In trueSpace, there is always and only one scene.
//    At the first level below (commonly referred to as the operational top
//       level) are instances of geometric objects (derived tsxGNODEs).
//    Geometric objects may be organized into tree-shaped hierarchies.
//       In such cases, only tsxGROUP or tsxLOD may occur as internal nodes
//       in the hierarchy. All other instances of GNODE types must occur at the
//       leaves of the hierarchy tree.
//    Each internal node, including the tsxSCENE node, points to
//       a list of its children.
//    Only GNODE types and tsxAXES may be selected
//       (i.e. made the "current object").
//    Geometric objects (GNODEs) may be positioned, rotated and scaled.  These
//       transformations may be done in either the World frame or the object's
//       own Model frame (see tsxFRAMEID).  The model frame is defined by the
//       position and orientation of the object's axes (see tsxGNode.h).
//    Only MNODE types allow their axes to be manipulated (tsxMNode.h).
//    At present not all trueSpace node types are supported in the TSX API.


// Object types in a scene.
enum tsxSOBJTYPE {
    e_tsxUNDEFINED = 0,	//Not a TSX-supported SOBJ type
    // SOBJ types
    e_tsxMATERIAL = 1,
    e_tsxSCENE,
    e_tsxPLUGIN_FILTER,
    e_tsxPHYS_WIND,
    e_tsxMATRECT,
    // Selectable objects
    e_tsxSELECTABLE = 50, //(Not a legal SOBJ type)
    e_tsxAXES,
    // GNODE types
    e_tsxGNODETYPES = 100, //(Not a legal SOBJ type)
    e_tsxCAMERA,
    e_tsxLIGHT,
    e_tsxJOINT,
    e_tsxNAIL,
    e_tsxLATTICE,
    e_tsxPATH,
    // MNODE types
    e_tsxMNODETYPES = 200, //(Not a legal SOBJ type)
    e_tsxPOLYHEDRON,
    e_tsxGROUP,
    e_tsxLODGROUP, //Levels of Detail
    e_tsxMBALLOBJ, //Object with meta-balls
//    e_tsxOPARTICLE,
    e_tsxSADEF, // Free-standing deformation
    e_tsxIKGROUP, //Object with joints (Inverse Kinematics)

    e_tsxSOBJEND //(Not a legal SOBJ type)
};


// The following are all really abstract type definitions,
// and their implementations are defined within TSXAPI.
// Use the specific create function to create an object
// of a particular type (e.g. tsxPolyhCreate for creating
// a Polyhedron).  Any object can be deleted with 
// tsxSobjDelete, though if some types have specific
// delete functions, then they should be used.
// Types that do not have a special function to create
// an instance cannot be created through the TSXAPI.
// The types labelled "Abstract type" below are abstractions
// of other types defined here.

struct TSXAPI tsxSOBJ{};	// Abstract type
// Derived from tsxSOBJ
struct TSXAPI tsxMATERIAL{};
struct TSXAPI tsxSCENE{};
struct TSXAPI tsxPLUGIN_FILTER{};
struct TSXAPI tsxPHYS_WIND{};
struct TSXAPI tsxMATRECT{};
struct TSXAPI tsxAXES{};

struct TSXAPI tsxGNODE{};	// Abstract type
// Derived from tsxGNODE
struct TSXAPI tsxCAMERA{};
struct TSXAPI tsxLIGHT{};
struct TSXAPI tsxJOINT{};
struct TSXAPI tsxNAIL{};
struct TSXAPI tsxLATTICE{};
struct TSXAPI tsxPATH{};

struct TSXAPI tsxMNODE{};	// Abstract type
// Derived from tsxMNODE
struct TSXAPI tsxPOLYHEDRON{};
struct TSXAPI tsxGROUP{};
struct TSXAPI tsxLODGROUP{};
struct TSXAPI tsxMBALLOBJ{};
//struct TSXAPI tsxOPARTICLE{};
struct TSXAPI tsxSADEF{};
struct TSXAPI tsxIKGROUP{};

// The following is abstract type definition,
// and its implementation is defined within TSXAPI.
// Use the specific create function to create a keyframe
// of a particular type (tsxFrameCreate). Any keyframe
// can be deleted with tsxFrameDelete.

struct TSXAPI tsxKEYFRAME{};

//------------------------------------------------------------------------------
//	Types for Callback Functions
//------------------------------------------------------------------------------

// Function with no args and no return value.
typedef void (*tsxBasicCallbackFP)(void);


//******************************************************************************
#endif // TSXTYPES_H
